import json


class SchemaMeta:
    def __init__(self, subject, version, id, schema):
        self.subject = subject
        self.version = version
        self.id = id
        if isinstance(schema, str):
            self.schema = json.loads(schema)
        elif isinstance(schema, dict):
            self.schema = schema
        else:
            raise Exception(
                "Unsupported schema type, expected `str` or `dict` but was {}".format(
                    type(schema)
                )
            )


class Schema:
    def __init__(self, schema, id):
        self.id = id
        self.schema = json.loads(schema)



import uuid
import time
from kafka import KafkaConsumer

from primed_avro.decoder import FastAvroDecoder
from primed_avro.registry import ConfluentSchemaRegistryClient


class AvroConsumer:
    def __init__(
        self,
        topic,
        registry_url,
        bootstrap_servers,
        group_id=str(uuid.uuid4()),
        auto_offset_reset="latest",
        max_poll_records=500,
    ):
        self._consumer = KafkaConsumer(
            topic,
            group_id=group_id,
            bootstrap_servers=bootstrap_servers,
            auto_offset_reset=auto_offset_reset,
            consumer_timeout_ms=10000,
            max_poll_records=max_poll_records,
        )

        self._registry_client = ConfluentSchemaRegistryClient(url=registry_url)
        self._decoder = FastAvroDecoder(schema_registry=self._registry_client)

    def consume(self, interval=1):
        while True:
            for msg in self._consumer:
                yield self._decoder.decode(msg.value)
            time.sleep(interval)



import avro
import fastavro
import struct
import io
import avro.io
from avro import schema as avroschema
import json
import primed_avro.util


MAGIC_BYTE = 0


class SerializerError(Exception):
    pass


class AvroDecoder:
    def __init__(self, schema):
        raise NotImplementedError()
        parsed_schema = avroschema.Parse(json.dumps(schema))
        self._writer = avro.io.DatumWriter(parsed_schema)

    def decode(self, record):
        writer_schema_obj = None  # TODO: where to obtain this?
        reader_schema_obj = None  # TODO: where to obtain this?

        if record is None:
            return None

        if len(record) <= 5:
            raise SerializerError("record is too small to decode")

        avro_reader = avro.io.DatumReader(writer_schema_obj, reader_schema_obj)

        with primed_avro.util.ContextStringIO(record) as payload:
            magic, schema_id = struct.unpack(">bI", payload.read(5))
            if magic != MAGIC_BYTE:
                raise SerializerError("record does not start with magic byte")

            return avro_reader.read(avro.io.BinaryDecoder(payload))


class FastAvroDecoder:
    def __init__(self, schema_registry, schemaless=True):
        self._schema_registry = schema_registry
        self._decoder = fastavro.schemaless_reader if schemaless else fastavro.reader

    def decode(self, record):

        with primed_avro.util.ContextStringIO(record) as buf:
            magic, schema_id = struct.unpack(">bI", buf.read(5))
            if magic != MAGIC_BYTE:
                raise SerializerError("record does not start with magic byte")

            schema = self._schema_registry.get_by_id(schema_id).schema

            return self._decoder(buf, schema)
    
    def decode_with_meta(self, record):

        with primed_avro.util.ContextStringIO(record) as buf:
            magic, schema_id = struct.unpack(">bI", buf.read(5))
            if magic != MAGIC_BYTE:
                raise SerializerError("record does not start with magic byte")

            schema = self._schema_registry.get_by_id(schema_id)

            return self._decoder(buf, schema.schema), schema, magic


class Decoder:
    """
    Allows the user to dynamically specify whether to use the
    FastAvro or regular Avro implementations
    """

    _classmap = {"fastavro": FastAvroDecoder, "avro": AvroDecoder}

    def __init__(self, schema, classname="fastavro"):
        self._decoder = Decoder._classmap[classname](schema)

    def get(self):
        return self._decoder



from setuptools import setup, find_packages
from codecs import open
from os import path
import sys

sys.path.insert(0, path.dirname(path.abspath(__file__)) + "/../")

import primed_avro


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# get the dependencies and installs
with open(path.join(here, "requirements.txt"), encoding="utf-8") as f:
    all_reqs = f.read().split("\n")

install_requires = [x.strip() for x in all_reqs if "git+" not in x]
dependency_links = [
    x.strip().replace("git+", "") for x in all_reqs if x.startswith("git+")
]

setup(
    name="primed-avro",
    version=primed_avro.__version__,
    description="A python package that provides functionality to interface with the Confluent Schema Registtry",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/primedio/primed-avro",
    download_url="https://gitlab.com/primedio/primed-avro/tarball/"
    + primed_avro.__version__,
    license="BSD",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
    ],
    keywords="",
    packages=find_packages(exclude=["docs", "tests*"]),
    include_package_data=True,
    author="Matthijs van der Kroon",
    install_requires=install_requires,
    dependency_links=dependency_links,
    author_email="matthijs@primed.io",
)



import requests
import json
from primed_avro.schema import SchemaMeta, Schema


class ConfluentSchemaRegistryClient:
    def __init__(self, url):
        self.url = url

        # keeps a local cache of requested schema's
        self._schema_cache = {}

    def get_schema(self, subject, version="latest"):
        assert isinstance(subject, str)

        r = requests.get(
            "{url}/subjects/{subject}/versions/{version}".format(
                url=self.url, subject=subject, version=version
            ),
            headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
        )

        if r.status_code == 200:
            js = r.json()

            if "error_code" in js:
                raise Exception(
                    "failed to retrieve schema, error_code: {error_code}, msg: {msg}".format(
                        error_code=js["error_code"], msg=js["msg"]
                    )
                )

            return SchemaMeta(**js)
        else:
            raise Exception(
                "failed to retrieve schema, status code: {status_code}".format(
                    status_code=r.status_code
                )
            )

    def create_schema(self, subject, schema):
        assert isinstance(schema, dict)
        assert isinstance(subject, str)

        r = requests.post(
            "{url}/subjects/{subject}/versions".format(url=self.url, subject=subject),
            headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
            data=json.dumps({"schema": json.dumps(schema)}),
        )

        if r.status_code == 200:
            js = r.json()

            return SchemaMeta(id=js["id"], subject=subject, version=None, schema=schema)
        else:
            raise Exception(
                "failed to upsert schema, status code: {status_code}: {text}".format(
                    status_code=r.status_code, text=r.text
                )
            )

    def get_by_id(self, id):
        # verify that we haven't already requested this
        # particular schema
        if id not in self._schema_cache:
            # http://localhost:8089/schemas/ids/23
            r = requests.get(
                "{url}/schemas/ids/{id}".format(url=self.url, id=id),
                headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
            )

            if r.status_code == 200:
                js = r.json()

                if "error_code" in js:
                    raise Exception(
                        "failed to retrieve schema, error_code: {error_code}, msg: {msg}".format(
                            error_code=js["error_code"], msg=js["msg"]
                        )
                    )

                self._schema_cache[id] = Schema(id=id, **js)
            else:
                raise Exception(
                    "failed to retrieve schema due to connection problems, status code: {status_code}".format(
                        status_code=r.status_code
                    )
                )

        return self._schema_cache[id]
